



SpoilsOfWar = LibStub("AceAddon-3.0"):NewAddon("SpoilsOfWar", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")


--@alpha@
local Version = "r".."17".." (alpha)";
--@end-alpha@

--[===[@non-alpha@
local Version = "r17"
--@end-non-alpha@]===]

local LootQueue = {};
local LootWaiting;

local options = {
	name = "SpoilsOfWar",
	handler = SpoilsOfWar,
    type = "group",
    args = {
		Reset = {
			type = "execute",
			name = "Reset",
			desc = "Reset Window Position",
			func = "ResetWindowPosition",
			order = 1,
		},
		Verbose = {
			type = "toggle",
			name = "Verbose",
			desc = "Toggles Verbose Mode",
			set = "ToggleVerbose",
			get = "GetVerbose",
			order = 3,
		},
		BBCode = {
			type = "toggle",
			name = "BBCode",
			desc = "Use BBCode when reporting items",
			set = "ToggleBBCode",
			get = "GetBBCode",
			order = 2,
		},
		ChatChannel = {
			type = "input",
			name = "Chat Channel",
			desc = "Chat Channel to report in for logging",
			set = function(info, v) SpoilsOfWar.db.profile.ChatChannel=strtrim(v) end,
			get = function() return SpoilsOfWar.db.profile.ChatChannel end,
		},
	},
}

local defaults = {
	profile = {
		Verbose = false,
	},
}

--[[
Wowhead List of Party Loot: http://www.wowhead.com/?items&filter=qu=2:3:4:5:6:7;cr=98;crs=1;crv=0
--]]

local PartyLootList = {30317,30316,30312,30318,30314,30313,30311,        	-- Legendary
                       43307,29434,40752,40753,43308,45624,					-- Epic
					   43876,32684,32683,9240,32682,32681,43228,38658,		-- Rare
					   11116,16408,10454,3985,8704,8623,8705,16304,			-- Uncommon
					   16305,7666,16782,16303}								-- Uncommon


function SpoilsOfWar:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SpoilsOfWarDB", defaults, "Default")
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SpoilsOfWar", options);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SpoilsOfWar", "SpoilsOfWar");
    self:RegisterChatCommand("spoils", "ChatCommand");
end



function SpoilsOfWar:OnLoad()
	VariablesLoaded = false;
	local x;
	
	_G["IgnoreButton"]:SetScript("OnClick",self.IgnoreLoot);
	SpoilsOfWar:RegisterEvent("VARIABLES_LOADED","OnEvent");
	SpoilsOfWar:RegisterEvent("CHAT_MSG_LOOT");
	SpoilsOfWar:RegisterEvent("CHAT_MSG_CHANNEL");
end

function SpoilsOfWar:OnEvent(event)
	if(event == "VARIABLES_LOADED") then
		VariablesLoaded = true;
	end
end

function SpoilsOfWar:CHAT_MSG_LOOT(event,arg1,...)
	arg1 = string.gsub(arg1,"You",UnitName("player"));
	
	if not string.find(arg1,"loot") then
		return;
	end
	
	SpoilsOfWar:FilterLoot(arg1);
end

local LastChatMessage
local LogStopTimerHandle
local TestTimerHandle

function SpoilsOfWar:CHAT_MSG_CHANNEL(event,msg,author,...)
	if msg == LastChatMessage and author == UnitName("player") then
		SpoilsOfWar:VerbosePrint("Found last chat message");
		
		local _,_,latency = GetNetStats();
		local timer = latency*1.20/1000;
		--timer = 0.75;
	
		--SpoilsOfWar:VerbosePrint("Starting fixed timer for: ".."0.75");
		SpoilsOfWar:VerbosePrint("Starting variable timer for: "..timer);
		LogStopTimerHandle = SpoilsOfWar:ScheduleTimer("LogStopTimer",timer);
		--TestTimerHandle = SpoilsOfWar:ScheduleTimer("TestTimerStop",timer);
	end
end

function SpoilsOfWar:ChatCommand(input)
	local x
	if not input or input:trim() == "" then
		SpoilsOfWar:ShowFrame();
	
	-- Needed to add a way to manually add loot.
	elseif string.find(input,"^add") then
		local _,_,name = string.find(input,"^add%s(.+)%s|c.+");
		local _,_,itemLink = string.find(input,"(|c%x+|H.+|h%[.*%])");
		if name and itemLink then
			table.insert(LootQueue,name.." receive loot: "..itemLink);
			if InCombatLockdown() then
				SpoilsOfWar:ForcePrint("Player in combat.  Queing loot.","ff0000");
			else
				SpoilsOfWar:ShowFrame();
			end
		else
			SpoilsOfWar:ForcePrint("Usage:  /spoils add <Name> <ItemLink>");
		end
	elseif string.find(input,"^marker") then
		local _,_,marker = string.find(input,"^marker%s(.+)");
		if marker then
			SpoilsOfWar:OutputChatMessage(marker);
		else
			SpoilsOfWar:ForcePrint("Usage:  /spoils marker <Marker Name>");
		end
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SpoilsOfWar, "spoils", "SpoilsOfWar", input)
	end
end

function SpoilsOfWar:LogStopTimer()
		LoggingChat(0);
		SpoilsOfWar:VerbosePrint("Logging Chat Off");
	if LogStopTimerHandle then
		SpoilsOfWar:CancelTimer("LogStopTimer",true);
		LogStopTimerHandle = nil;
	end
end

function SpoilsOfWar:TestTimerStop()
	SpoilsOfWar:VerbosePrint("Logging Chat Off (Variable Timer)");
	
	if TestTimerHandle then
		SpoilsOfWar:CancelTimer("TestTimerStop",true);
		TestTimerHandle = nil;
	end
end

function SpoilsOfWar:FilterLoot(arg1)
	if GetLootMethod() ~= "master" then
		return;
		-- we only care about master loot right now
	end

	local found,_,itemLink = string.find(arg1,"(|c%x+|H.+|h%[.*%])")
	
	if not found then
		return;
	end
	
	local itemName, _, itemRarity, _, _, _, _, _, _, itemTexture = GetItemInfo(itemLink);
	local _,_,itemID = string.find(itemLink,"^|c%x+|Hitem:(%d+).+");
	
	SpoilsOfWar:VerbosePrint("ItemID: "..itemID);
	
	-- Check item against party loot list.
	-- If lootable by entire party/raid, discard.
	local x;
	for x=1,#(PartyLootList) do
		if itemID == tostring(PartyLootList[x]) then
			return;
		end
	end
	
	_G["ScanningTooltip"]:ClearLines();
	_G["ScanningTooltip"]:SetHyperlink(itemLink);
	local lines = _G["ScanningTooltip"]:NumLines();
	
	local x,BindOn;
	BindOn = "Never";
	for x=1,lines do
		if string.find(_G["ScanningTooltipTextLeft"..x]:GetText(),"Binds when equipped") then
			BindOn = "Equip";
		elseif string.find(_G["ScanningTooltipTextLeft"..x]:GetText(),"Binds when picked up") then
			BindOn = "Pickup";
		elseif string.find(_G["ScanningTooltipTextLeft"..x]:GetText(),"Binds to account") then
			BindOn = "Account";
		end
	end
	
	-- Only add if it is >= the threshold for master looting, and >= Uncommon
	if (itemRarity >= GetLootThreshold()) and (itemRarity >= 2) then
		-- Queue all the incoming loot in case user isn't fast enough.
		table.insert(LootQueue,arg1);
		if InCombatLockdown() then
			SpoilsOfWar:ForcePrint("Player in combat.  Queing loot.","ff0000");
		else
			SpoilsOfWar:ShowFrame();
		end
	end
end

local CurrentLoot

function SpoilsOfWar:DisplayLoot()
	-- If we're already waiting on loot, don't display next one.
	if LootWaiting then
		return
	else
		--SpoilsOfWar:ShowFrame();
	end;
	
	
	-- Pull out first in Queue
	CurrentLoot = table.remove(LootQueue,1);
	
	if CurrentLoot == nil then
		_G["ItemNameLabel"]:SetText("");
		_G["ItemIconTexture"]:SetTexture("");
		_G["LooterNameLabel"]:SetText("");
		SpoilsOfWar:HideFrame();
		return;
	end
	
	local found,_,itemLink = string.find(CurrentLoot,"(|c%x+|H.+|h%[.*%])")
	local _, _, Looter = string.find(CurrentLoot,"(.+) receive")
	
	if found and itemLink and Looter then
		LootWaiting = true;
		SpoilsOfWar:VerbosePrint("Found loot: "..itemLink.."|r - "..Looter);
		local itemName, _, itemRarity, _, _, _, _, _, _, itemTexture = GetItemInfo(itemLink);
		local r,g,b = GetItemQualityColor(itemRarity);
		_G["ItemNameLabel"]:SetText(wrap("["..itemName.."]",18));
		_G["ItemNameLabel"]:SetTextColor(r,g,b,1);
		_G["ItemIconTexture"]:SetTexture(itemTexture);
		_G["LooterNameLabel"]:SetText(Looter);
	end
end

function wrap(str, limit, indent, indent1)
	indent = indent or ""
	indent1 = indent1 or indent
	limit = limit or 72
	local here = 1-#indent1
	return indent1..str:gsub("(%s+)()(%S+)()",
                          function(sp, st, word, fi)
                            if fi-here > limit then
                              here = st - #indent
                              return "\n"..indent..word
                            end
                          end)
end


function SpoilsOfWar:OutputChatMessage(message)
	local ReportChannelName, ReportChannelName;
	
	if (SpoilsOfWar.db.profile.ChatChannel == "RAID") then
		ReportChannelNumber = nil;
		ReportChannelName = "RAID"
	else	
		ReportChannelNumber,_ = GetChannelName(SpoilsOfWar.db.profile.ChatChannel);
		ReportChannelName = "CHANNEL"
	end
	
	if LogStopTimerHandle then
		SpoilsOfWar:CancelTimer("LogStopTimer",true);
		LogStopTimerHandle = nil;
	end
	
	if TestTimerHandle then
		SpoilsOfWar:CancelTimer("TestTimerStop",true);
		TestTimerHandle = nil;
	end
	
	LastChatMessage = message;
	LoggingChat(1);
	SpoilsOfWar:VerbosePrint("Logging Chat On");
	SendChatMessage(LastChatMessage,ReportChannelName,nil,ReportChannelNumber);
end

function SpoilsOfWar:LogLoot(TheWho,TheWhat,TheBy,TheWhy)	
	local itemName,itemID,LootMessage;
	
	_,_,itemID = string.find(TheWhat,"^|c%x+|Hitem:(%d+).+")
	itemName = GetItemInfo(TheWhat);
	
	if SpoilsOfWar.db.profile.BBCode then
		LootMessage = (TheWho.." awarded [url=http://www.wowhead.com/?item="..itemID.."]["..itemName.."][/url] by "..TheBy.." for "..TheWhy);
	else
		LootMessage = (TheWho.." awarded "..itemName.." by "..TheBy.." for "..TheWhy);
	end
	SpoilsOfWar:OutputChatMessage(LootMessage);
end

function SpoilsOfWar:RecordLoot(arg1,arg2)
	if CurrentLoot ~= nil then
		local found,_,itemLink = string.find(CurrentLoot,"(|c%x+|H.+|h%[.*%])");
		local _, _, Looter = string.find(CurrentLoot,"^(.+)%sreceive");
		local itemName, _, itemRarity, _, _, _, _, _, _, itemTexture = GetItemInfo(itemLink);
		
		SpoilsOfWar:LogLoot(Looter:trim(),itemLink,arg1,arg2);
	end
	
	LootWaiting = false;
	SpoilsOfWar:DisplayLoot();
end

function SpoilsOfWar:IgnoreLoot()	
	LootWaiting = false;
	SpoilsOfWar:DisplayLoot();
end

function SpoilsOfWar:ShowFrame()
	_G["SpoilsOfWarFrame"]:Show();
	_G["VersionTextLabel"]:SetText(Version);
	SpoilsOfWar:DisplayLoot();
	if (#(LootQueue) > 0) and not LoggingChat() then
		--LoggingChat(1);
		--SpoilsOfWar:VerbosePrint("Logging Chat On");
	end
end

function SpoilsOfWar:HideFrame()
	_G["SpoilsOfWarFrame"]:Hide();
end

function SpoilsOfWar:ResetWindowPosition()
	SpoilsOfWarFrame:ClearAllPoints();
	SpoilsOfWarFrame:SetPoint("CENTER","UIParent");
end

function SpoilsOfWar:ToggleVerbose()
	self.db.profile.Verbose = not self.db.profile.Verbose
end

function SpoilsOfWar:GetVerbose()
	return self.db.profile.Verbose;
end

function SpoilsOfWar:ToggleBBCode()
	self.db.profile.BBCode = not self.db.profile.BBCode
end

function SpoilsOfWar:GetBBCode()
	return self.db.profile.BBCode;
end




function SpoilsOfWar:VerbosePrint(msg,color)
	if (self.db.profile.Verbose) then
		if color ~= nil then
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Spoils of War: |cff"..color..msg.."|r");
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Spoils of War: |cffffffff"..msg.."|r");
		end
	end
end

function SpoilsOfWar:ForcePrint(msg,color)
	if color ~= nil then
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Spoils of War: |cff"..color..msg.."|r");
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Spoils of War: |cffffffff"..msg.."|r");
	end
end



local LDB = LibStub("LibDataBroker-1.1", true)
if LDB then
	local L_SpoilsOfWar_LEFT = "Left click to open setup window"
	local L_SpoilsOfWar_RIGHT = ""

	LibStub("LibDataBroker-1.1"):NewDataObject("SoW", {
		type = "launcher",
		text = "SoW",
		OnClick = function(_, msg)
			if msg == "LeftButton" then
				if _G["SpoilsOfWarFrame"]:IsShown() then
					SpoilsOfWar:HideFrame()
				else
					SpoilsOfWar:ShowFrame()
				end
			elseif msg == "RightButton" then
				--nothing yet
			end
		end,
		icon = "Interface\\Icons\\INV_Misc_Note_01",
		OnTooltipShow = function(tooltip)
			if not tooltip or not tooltip.AddLine then return end
			tooltip:AddLine("SpoilsOfWar")
			tooltip:AddLine(" ");
			tooltip:AddLine(L_SpoilsOfWar_LEFT)
			--tooltip:AddLine(L_SpoilsOfWar_RIGHT)
			tooltip:AddLine(" ");
			if #(LootQueue) > 0 then
				tooltip:AddLine("Loot queued: "..#(LootQueue))
			else
				tooltip:AddLine("Loot queue empty")
			end
		end,
	})
end